@@ -407,8 +407,9 @@ class ShardReader(object):
         results[label] = None
       else:
         if return_byte_range:
-          _, offset, size = minishard_index[idx,:]
-          results[label] = [ filepath, offset, size ]
+          _, offset, size = minishard_index[idx,:][0]
+          index_offset = self.spec.index_length()
+          results[label] = [ filepath, int(offset + index_offset), int(size) ]
         else:
           results[label] = filepath
 
